@@ -54,6 +54,15 @@ void ogs_nnrf_nfm_handle_nf_register(
         OpenAPI_list_for_each(NFProfile->plmn_list, node) {
             OpenAPI_plmn_id_t *PlmnId = node->data;
             if (PlmnId) {
+                if (ogs_local_conf()->num_of_serving_plmn_id >=
+                        OGS_ARRAY_SIZE(ogs_local_conf()->serving_plmn_id)) {
+                    ogs_error("OVERFLOW NFProfile->plmn_list [%d:%d:%d]",
+                            ogs_local_conf()->num_of_serving_plmn_id,
+                            OGS_MAX_NUM_OF_PLMN,
+                            (int)OGS_ARRAY_SIZE(
+                                ogs_local_conf()->serving_plmn_id));
+                    break;
+                }
                 ogs_sbi_parse_plmn_id(
                     &ogs_local_conf()->serving_plmn_id[
                         ogs_local_conf()->num_of_serving_plmn_id], PlmnId);
@@ -96,6 +105,13 @@ void ogs_nnrf_nfm_handle_nf_profile(
     OpenAPI_list_for_each(NFProfile->plmn_list, node) {
         OpenAPI_plmn_id_t *PlmnId = node->data;
         if (PlmnId) {
+            if (nf_instance->num_of_plmn_id >=
+                    OGS_ARRAY_SIZE(nf_instance->plmn_id)) {
+                ogs_error("OVERFLOW NFProfile->plmn_list [%d:%d:%d]",
+                        nf_instance->num_of_plmn_id, OGS_MAX_NUM_OF_PLMN,
+                        (int)OGS_ARRAY_SIZE(nf_instance->plmn_id));
+                break;
+            }
             ogs_sbi_parse_plmn_id(
                 &nf_instance->plmn_id[nf_instance->num_of_plmn_id], PlmnId);
             nf_instance->num_of_plmn_id++;
@@ -467,8 +483,12 @@ static void handle_smf_info(
         TaiRangeItem = node->data;
         if (TaiRangeItem && TaiRangeItem->plmn_id &&
                 TaiRangeItem->tac_range_list) {
-            ogs_assert(nf_info->smf.num_of_nr_tai_range <
-                    OGS_MAX_NUM_OF_TAI);
+
+            if (nf_info->smf.num_of_nr_tai_range >= OGS_MAX_NUM_OF_TAI) {
+                ogs_error("OVERFLOW TaiRangeItem [%d:%d]",
+                        nf_info->smf.num_of_nr_tai_range, OGS_MAX_NUM_OF_TAI);
+                break;
+            }
 
             ogs_sbi_parse_plmn_id(
                 &nf_info->smf.nr_tai_range
@@ -709,7 +729,13 @@ static void handle_amf_info(
         TaiItem = node->data;
         if (TaiItem && TaiItem->plmn_id && TaiItem->tac) {
             ogs_5gs_tai_t *nr_tai = NULL;
-            ogs_assert(nf_info->amf.num_of_nr_tai < OGS_MAX_NUM_OF_TAI);
+
+            if (nf_info->amf.num_of_nr_tai >= OGS_MAX_NUM_OF_TAI) {
+                ogs_error("OVERFLOW TaiItem [%d:%d]",
+                        nf_info->amf.num_of_nr_tai, OGS_MAX_NUM_OF_TAI);
+                break;
+            }
+
             nr_tai = &nf_info->amf.nr_tai[nf_info->amf.num_of_nr_tai];
             ogs_assert(nr_tai);
             ogs_sbi_parse_plmn_id(&nr_tai->plmn_id, TaiItem->plmn_id);
@@ -726,6 +752,12 @@ static void handle_amf_info(
             ogs_assert(nf_info->amf.num_of_nr_tai_range <
                     OGS_MAX_NUM_OF_TAI);
 
+            if (nf_info->amf.num_of_nr_tai_range >= OGS_MAX_NUM_OF_TAI) {
+                ogs_error("OVERFLOW TaiRangeItem [%d:%d]",
+                        nf_info->amf.num_of_nr_tai_range, OGS_MAX_NUM_OF_TAI);
+                break;
+            }
+
             ogs_sbi_parse_plmn_id(
                 &nf_info->amf.nr_tai_range
                     [nf_info->amf.num_of_nr_tai_range].plmn_id,
@@ -824,6 +856,17 @@ void ogs_nnrf_nfm_handle_nf_status_subscribe(
 {
     OpenAPI_subscription_data_t *SubscriptionData = NULL;
 
+    int rv;
+    ogs_sbi_message_t message;
+    ogs_sbi_header_t header;
+
+    bool rc;
+    ogs_sbi_client_t *client = NULL;
+    OpenAPI_uri_scheme_e scheme = OpenAPI_uri_scheme_NULL;
+    char *fqdn = NULL;
+    uint16_t fqdn_port = 0;
+    ogs_sockaddr_t *addr = NULL, *addr6 = NULL;
+
     ogs_assert(recvmsg);
     ogs_assert(subscription_data);
 
@@ -833,45 +876,63 @@ void ogs_nnrf_nfm_handle_nf_status_subscribe(
         return;
     }
 
-    if (recvmsg->http.location) {
-        int rv;
-        ogs_sbi_message_t message;
-        ogs_sbi_header_t header;
-
-        memset(&header, 0, sizeof(header));
-        header.uri = recvmsg->http.location;
-
-        rv = ogs_sbi_parse_header(&message, &header);
-        if (rv != OGS_OK) {
-            ogs_error("Cannot parse http.location [%s]",
-                recvmsg->http.location);
-            return;
-        }
-
-        if (!message.h.resource.component[1]) {
-            ogs_error("No Subscription ID [%s]", recvmsg->http.location);
-            ogs_sbi_header_free(&header);
-            return;
-        }
-
-        ogs_sbi_subscription_data_set_id(
-                subscription_data, message.h.resource.component[1]);
-
-        ogs_sbi_header_free(&header);
-
-    } else if (SubscriptionData->subscription_id) {
-        /*
-         * For compatibility with v2.5.x and lower versions
-         *
-         * Deprecated : It will be removed soon.
-         */
-        ogs_sbi_subscription_data_set_id(
-            subscription_data, SubscriptionData->subscription_id);
-    } else {
-        ogs_error("No Subscription ID");
+    if (!recvmsg->http.location) {
+        ogs_error("No http.location");
         return;
     }
 
+    memset(&header, 0, sizeof(header));
+    header.uri = recvmsg->http.location;
+
+    rv = ogs_sbi_parse_header(&message, &header);
+    if (rv != OGS_OK) {
+        ogs_error("Cannot parse http.location [%s]",
+            recvmsg->http.location);
+        return;
+    }
+
+    if (!message.h.resource.component[1]) {
+        ogs_error("No Subscription ID [%s]", recvmsg->http.location);
+        ogs_sbi_header_free(&header);
+        return;
+    }
+
+    rc = ogs_sbi_getaddr_from_uri(
+            &scheme, &fqdn, &fqdn_port, &addr, &addr6, header.uri);
+    if (rc == false || scheme == OpenAPI_uri_scheme_NULL) {
+        ogs_error("Invalid URI [%s]", header.uri);
+        ogs_sbi_header_free(&header);
+        return;
+    }
+
+    client = ogs_sbi_client_find(scheme, fqdn, fqdn_port, addr, addr6);
+    if (!client) {
+        ogs_debug("%s: ogs_sbi_client_add()", OGS_FUNC);
+        client = ogs_sbi_client_add(scheme, fqdn, fqdn_port, addr, addr6);
+        if (!client) {
+            ogs_error("%s: ogs_sbi_client_add() failed", OGS_FUNC);
+
+            ogs_sbi_header_free(&header);
+            ogs_free(fqdn);
+            ogs_freeaddrinfo(addr);
+            ogs_freeaddrinfo(addr6);
+
+            return;
+        }
+    }
+    OGS_SBI_SETUP_CLIENT(subscription_data, client);
+
+    ogs_free(fqdn);
+    ogs_freeaddrinfo(addr);
+    ogs_freeaddrinfo(addr6);
+
+    ogs_sbi_subscription_data_set_resource_uri(
+            subscription_data, header.uri);
+    ogs_sbi_subscription_data_set_id(
+            subscription_data, message.h.resource.component[1]);
+
+    ogs_sbi_header_free(&header);
+
     /* SBI Features */
     if (SubscriptionData->nrf_supported_features) {
         subscription_data->nrf_supported_features =
@@ -1026,24 +1087,23 @@ bool ogs_nnrf_nfm_handle_nf_status_notify(
                     nf_instance, message.h.resource.component[1]);
             ogs_sbi_nf_fsm_init(nf_instance);
 
-            ogs_info("(NRF-notify) NF registered [%s:%d]",
-                    nf_instance->id, nf_instance->reference_count);
+            ogs_info("[%s] (NRF-notify) NF registered", nf_instance->id);
         } else {
-            ogs_warn("[%s] (NRF-notify) NF has already been added [%s:%d]",
-                    nf_instance->nf_type ?
-                        OpenAPI_nf_type_ToString(nf_instance->nf_type) : "NULL",
-                    nf_instance->id, nf_instance->reference_count);
-
-            ogs_assert(OGS_FSM_STATE(&nf_instance->sm));
-            ogs_sbi_nf_fsm_tran(nf_instance, ogs_sbi_nf_state_registered);
+            ogs_warn("[%s] (NRF-notify) NF has already been added [type:%s]",
+                    nf_instance->id,
+                    OpenAPI_nf_type_ToString(nf_instance->nf_type));
+            if (!OGS_FSM_CHECK(&nf_instance->sm, ogs_sbi_nf_state_registered)) {
+                ogs_error("[%s] (NRF-notify) NF invalid state [type:%s]",
+                        nf_instance->id,
+                        OpenAPI_nf_type_ToString(nf_instance->nf_type));
+            }
         }
 
         ogs_nnrf_nfm_handle_nf_profile(nf_instance, NFProfile);
 
-        ogs_info("[%s] (NRF-notify) NF Profile updated [%s:%d]",
-                    nf_instance->nf_type ?
-                        OpenAPI_nf_type_ToString(nf_instance->nf_type) : "NULL",
-                    nf_instance->id, nf_instance->reference_count);
+        ogs_info("[%s] (NRF-notify) NF Profile updated [type:%s]",
+                    nf_instance->id,
+                    OpenAPI_nf_type_ToString(nf_instance->nf_type));
 
         ogs_sbi_client_associate(nf_instance);
 
@@ -1059,27 +1119,11 @@ bool ogs_nnrf_nfm_handle_nf_status_notify(
             OpenAPI_notification_event_type_NF_DEREGISTERED) {
         nf_instance = ogs_sbi_nf_instance_find(message.h.resource.component[1]);
         if (nf_instance) {
-            if (OGS_OBJECT_IS_REF(nf_instance)) {
-                /* There are references to other contexts. */
-                ogs_warn("[%s] (NRF-notify) NF was referenced "
-                        "in other contexts [%s:%d]",
-                        nf_instance->nf_type ?
-                            OpenAPI_nf_type_ToString(nf_instance->nf_type) :
-                            "NULL",
-                        nf_instance->id, nf_instance->reference_count);
-
-                ogs_assert(OGS_FSM_STATE(&nf_instance->sm));
-                ogs_sbi_nf_fsm_tran(
-                        nf_instance, ogs_sbi_nf_state_de_registered);
-            } else {
-                ogs_info("[%s] (NRF-notify) NF_DEREGISTERED event [%s:%d]",
-                        nf_instance->nf_type ?
-                            OpenAPI_nf_type_ToString(nf_instance->nf_type) :
-                            "NULL",
-                        nf_instance->id, nf_instance->reference_count);
-                ogs_sbi_nf_fsm_fini((nf_instance));
-                ogs_sbi_nf_instance_remove(nf_instance);
-            }
+            ogs_info("[%s] (NRF-notify) NF_DEREGISTERED event [type:%s]",
+                    nf_instance->id,
+                    OpenAPI_nf_type_ToString(nf_instance->nf_type));
+            ogs_sbi_nf_fsm_fini(nf_instance);
+            ogs_sbi_nf_instance_remove(nf_instance);
         } else {
             ogs_warn("[%s] (NRF-notify) Not found",
                     message.h.resource.component[1]);
@@ -1155,18 +1199,18 @@ void ogs_nnrf_disc_handle_nf_discover_search_result(
             ogs_sbi_nf_instance_set_id(nf_instance, NFProfile->nf_instance_id);
             ogs_sbi_nf_fsm_init(nf_instance);
 
-            ogs_info("[%s] (NRF-discover) NF registered [%s:%d]",
-                    nf_instance->nf_type ?
-                        OpenAPI_nf_type_ToString(nf_instance->nf_type) : "NULL",
-                    nf_instance->id, nf_instance->reference_count);
+            ogs_info("[%s] (NRF-discover) NF registered [type:%s]",
+                    nf_instance->id,
+                    OpenAPI_nf_type_ToString(nf_instance->nf_type));
         } else {
-            ogs_warn("[%s] (NRF-discover) NF has already been added [%s:%d]",
-                    nf_instance->nf_type ?
-                        OpenAPI_nf_type_ToString(nf_instance->nf_type) : "NULL",
-                    nf_instance->id, nf_instance->reference_count);
-
-            ogs_assert(OGS_FSM_STATE(&nf_instance->sm));
-            ogs_sbi_nf_fsm_tran(nf_instance, ogs_sbi_nf_state_registered);
+            ogs_warn("[%s] (NRF-discover) NF has already been added [type:%s]",
+                    nf_instance->id,
+                    OpenAPI_nf_type_ToString(nf_instance->nf_type));
+            if (!OGS_FSM_CHECK(&nf_instance->sm, ogs_sbi_nf_state_registered)) {
+                ogs_error("[%s] (NRF-notify) NF invalid state [type:%s]",
+                        nf_instance->id,
+                        OpenAPI_nf_type_ToString(nf_instance->nf_type));
+            }
         }
 
         if (NF_INSTANCE_ID_IS_OTHERS(nf_instance->id)) {
@@ -1193,15 +1237,18 @@ void ogs_nnrf_disc_handle_nf_discover_search_result(
                     ogs_time_from_sec(nf_instance->time.validity_duration));
 
             } else
-                ogs_warn("[%s] NF Instance validity-time should not 0 [%s:%d]",
+                ogs_warn("[%s] NF Instance validity-time should not 0 "
+                        "[type:%s]",
+                    nf_instance->id,
                     nf_instance->nf_type ?
-                        OpenAPI_nf_type_ToString(nf_instance->nf_type) : "NULL",
-                    nf_instance->id, nf_instance->reference_count);
+                        OpenAPI_nf_type_ToString(nf_instance->nf_type) :
+                        "NULL");
 
-            ogs_info("[%s] (NF-discover) NF Profile updated [%s:%d]",
-                    nf_instance->nf_type ?
-                        OpenAPI_nf_type_ToString(nf_instance->nf_type) : "NULL",
-                    nf_instance->id, nf_instance->reference_count);
+            ogs_info("[%s] (NF-discover) NF Profile updated "
+                    "[type:%s validity:%ds]",
+                    nf_instance->id,
+                    OpenAPI_nf_type_ToString(nf_instance->nf_type),
+                    nf_instance->time.validity_duration);
         }
     }
 }
